package timelineservicelogic

import (
	"app-cloudep-tweeting-service/internal/domain"
	"context"

	ers "code.30cm.net/digimon/library-go/errs"

	"app-cloudep-tweeting-service/gen_result/pb/tweeting"
	"app-cloudep-tweeting-service/internal/svc"

	"github.com/zeromicro/go-zero/core/logx"
)

type SetNoMoreDataFlagLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewSetNoMoreDataFlagLogic(ctx context.Context, svcCtx *svc.ServiceContext) *SetNoMoreDataFlagLogic {
	return &SetNoMoreDataFlagLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

type sasNoMoreDataReq struct {
	UID string `json:"uid" validate:"required"`
}

// SetNoMoreDataFlag 標記時間線已完整，避免繼續查詢資料庫。
func (l *SetNoMoreDataFlagLogic) SetNoMoreDataFlag(in *tweeting.DoNoMoreDataReq) (*tweeting.OKResp, error) {
	// 驗證資料
	if err := l.svcCtx.Validate.ValidateAll(&sasNoMoreDataReq{
		UID: in.GetUid(),
	}); err != nil {
		// 錯誤代碼 05-011-00
		return nil, ers.InvalidFormat(err.Error())
	}

	err := l.svcCtx.TimelineRepo.SetNoMoreDataFlag(l.ctx, in.GetUid())
	if err != nil {
		// 錯誤代碼 05-021-24
		e := domain.CommentErrorL(
			domain.SetNoMoreDataErrorCode,
			logx.WithContext(l.ctx),
			[]logx.LogField{
				{Key: "req", Value: in},
				{Key: "func", Value: "TimelineRepo.SetNoMoreDataErrorCode"},
				{Key: "err", Value: err},
			},
			"failed to set no more data flag:", in.GetUid()).Wrap(err)
		return nil, e
	}

	return &tweeting.OKResp{}, nil
}
